﻿using HarmonyLib;
using System;
using System.Collections.Generic;
using thmsn.CraftFromAllStorage.Extensions;
using thmsn.CraftFromAllStorage.Network;
using UnityEngine;

namespace thmsn.CraftFromAllStorage
{
    class CraftFromStorageManager
    {
        // TODO: could move to RAPI extension
        static public bool HasUnlimitedResources()
        {
            return GameModeValueManager.GetCurrentGameModeValue().playerSpecificVariables.unlimitedResources;
        }

        /// <summary>
        /// costBox.items is protected on BuildingUI_CostBox, so we use harmony to get access
        /// </summary>
        /// <param name="costBox"></param>
        /// <returns></returns>
        static public List<Item_Base> getItemsFromCostBox(BuildingUI_CostBox costBox)
        {
            return Traverse.Create(costBox).Field("items").GetValue<List<Item_Base>>();
        }

        static public void RemoveCostMultiple(CostMultiple[] costMultipleArray, bool manipulateCostAmount = false)
        {
            if (!HasUnlimitedResources())
            {
                //Debug.Log("Getting player inventory");
                var playerInventory = InventoryManager.GetPlayerInventory();

                var itemsAndAmountToRemove = new List<ItemNameAndAmount>();
                var matches = new Dictionary<CostMultiple, Ref<int>>();

                // Remove items from player inventory, then the open storage, then other chests.
                foreach (var costMultiple in costMultipleArray)
                {
                    if (costMultiple.amount <= 0)
                        continue;

                    var amount = matches[costMultiple] = costMultiple.amount;
                    // Usually there is only 1 item in .items. for example Plank
                    foreach (var item in costMultiple.items)
                        itemsAndAmountToRemove.Add(new ItemNameAndAmount(item.UniqueName, amount));

                }

                playerInventory.RemoveItemFromPlayerInventoryAndStoragesOnRaft(itemsAndAmountToRemove);

                if (manipulateCostAmount)
                    foreach (var pair in matches)
                        pair.Key.amount = pair.Value;
            }
        }


    }
}