﻿using HarmonyLib;
using thmsn.CraftFromAllStorage.Extensions;
using thmsn.CraftFromAllStorage.Network;
using System.Collections.Generic;
using System.Reflection.Emit;
using System.Linq;
using System.Reflection;

namespace thmsn.CraftFromAllStorage.Patches
{
    /// <summary>
    /// This needs to be overriden to indicate we have enough resources
    /// Handle both fuel counts and item counts from AutoRecipeBehaviour.CalculatePreparation (colors resources red if missing)
    /// Also prevent duplicate slots from being counted as some modded storages list every slot twice
    /// </summary>
    [HarmonyPatch(typeof(Inventory), nameof(Inventory.GetItemCount), typeof(Item_Base))] // TODO: Extract out, it is no longer recipe redux responsible, it is also responsible for Block_CookingStand_Smelter
    class Patch_PlayerInventory_GetItemCount
    {
        static void Postfix(Inventory __instance, ref int __result, Item_Base item)
        {
            if (item && __instance && __instance is PlayerInventory playerInventory)
            {
                if (playerInventory.secondInventory != null)
                    __result += playerInventory.secondInventory.GetItemCount(item.UniqueName);

                if (!CraftFromStorageManager.HasUnlimitedResources())
                    foreach (Storage_Small storage in StorageManager.allStorages)
                    {
                        if (!storage || storage.IsExcludeFromCraftFromAllStorage() || storage.IsOpen)
                            continue;
                        var container = storage.GetInventoryReference();
                        if (container)
                            __result += container.GetItemCount(item.UniqueName);
                    }
            }
        }

        public static IEnumerable<CodeInstruction> Transpiler(IEnumerable<CodeInstruction> instructions, ILGenerator iL)
        {
            var loc = iL.DeclareLocal(typeof(CountManager));
            var code = instructions.ToList();
            code.InsertRange(0, new[]
            {
                new CodeInstruction(OpCodes.Call, AccessTools.Method(typeof(CountManager),nameof(CountManager.Create))),
                new CodeInstruction(OpCodes.Stloc_S, loc)
            });
            var ind = code.FindIndex(x => x.operand is MethodInfo method && method.DeclaringType == typeof(List<Slot>.Enumerator) && method.Name == "MoveNext");
            code.InsertRange(ind + 1, new[] {
                new CodeInstruction(code[ind - 1]),
                new CodeInstruction(OpCodes.Ldloc_S,loc),
                new CodeInstruction(OpCodes.Call,AccessTools.Method(typeof(Patch_PlayerInventory_GetItemCount),nameof(OverrideMoveNext)))
                });
            return code;
        }
        static bool OverrideMoveNext(bool original, ref List<Slot>.Enumerator enumerator, CountManager counter)
        {
            if (!original)
                return original;
            while (!counter.ShouldCount(enumerator.Current))
                if (!enumerator.MoveNext())
                    return false;
            return true;
        }
        class CountManager
        {
            HashSet<Slot> counted = new HashSet<Slot>();
            public bool ShouldCount(Slot slot) => counted.Add(slot);
            public static CountManager Create() => new CountManager();
        }
    }
    [HarmonyPatch(typeof(Inventory), nameof(Inventory.GetItemCount), typeof(string))]
    class Patch_PlayerInventory_GetItemCount2
    {
        static IEnumerable<CodeInstruction> Transpiler(IEnumerable<CodeInstruction> instructions, ILGenerator iL) => Patch_PlayerInventory_GetItemCount.Transpiler(instructions, iL);
    }
}